@@ -1,9 +1,10 @@
-from os import chdir, getcwd
-from subprocess import call, PIPE
-from commandlib.utils import _check_directory
 from commandlib.exceptions import CommandError
-import io
+from commandlib.utils import _check_directory
+from commandlib.piped import PipedCommand
+from subprocess import PIPE, Popen
+from os import chdir, getcwd
 import copy
+import sys
 import os
 
 
@@ -48,6 +49,7 @@ class Command(object):
         self._pipe_stdout_to_file = None
         self._pipe_stderr_to_file = None
         self._pipe_from_file = None
+        self._pipe_from_string = None
 
     @property
     def arguments(self):
@@ -64,7 +66,7 @@ class Command(object):
         env_vars = os.environ.copy()
         env_vars.update(self._env)
         new_path = ":".join(
-            [env_vars["PATH"], ] + self._paths if "PATH" in env_vars else [] + self._paths
+            self._paths + [env_vars["PATH"], ] if "PATH" in env_vars else [] + self._paths
         )
         env_vars["PATH"] = new_path
         for env_var in self._env_drop:
@@ -208,26 +210,62 @@ class Command(object):
 
     def pipe_from_file(self, handle):
         """
-        Pipe the stderr output to file handle 'handle'.
+        Pipe the contents of the handle 'handle' to the command.
 
         Example usage::
-          command.pipe_stderr_to_file(open("/tmp/output", 'w'))
+          command.pipe_from_file(open("/tmp/output", 'w'))
         """
         new_command = copy.deepcopy(self)
         new_command._pipe_from_file = handle
         return new_command
 
+    def pipe_from_string(self, string):
+        """
+        Pipe the contents of the string to the command's stdin.
+
+        Example usage::
+          command.pipe_from_string("hello")
+        """
+        new_command = copy.deepcopy(self)
+        new_command._pipe_from_string = string
+        return new_command
+
+    def interact(self):
+        """
+        Return icommand object which you can then run.
+
+        NOTE: Requires you to pip install 'icommandlib' or will fail.
+        """
+        import icommandlib
+        return icommandlib.ICommand(self)
+
+    @property
+    def piped(self):
+        """
+        Return PipedCommand object.
+
+        This is what you use if you want to do limited piped
+        interactions with the command:
+
+        * Pipe a file handle in or out of the command.
+        * Pipe in a string.
+        * Run and get the output of the command.
+        """
+        return PipedCommand(self)
+
     def pexpect(self):
         """
         Run command and return pexpect process object.
+
+        NOTE: Requires you to pip install 'pexpect' or will fail.
         """
         import pexpect
         assert self._pipe_stderr_to_file is None
         assert self._pipe_stdout_to_file is None
         assert self._pipe_from_file is None
-        assert self._silent_stdout == False
-        assert self._silent_stderr == False
-        assert self._ignore_errors == False
+        assert not self._silent_stdout
+        assert not self._silent_stderr
+        assert not self._ignore_errors
 
         _check_directory(self.directory)
 
@@ -271,17 +309,30 @@ class Command(object):
         if self._silent_stderr:
             stderr = PIPE
 
-        stdin = None if self._pipe_from_file is None else self._pipe_from_file
+        if self._pipe_from_file is None and self._pipe_from_string is None:
+            stdin = None
+        else:
+            if self._pipe_from_string:
+                stdin = PIPE
+            if self._pipe_from_file:
+                stdin = self._pipe_from_file
 
-        returncode = call(
+        process = Popen(
             self.arguments,
-            env=self.env,
-            shell=self._shell,
             stdout=stdout,
             stderr=stderr,
             stdin=stdin,
+            shell=self._shell,
+            env=self.env,
         )
 
+        if self._pipe_from_string:
+            process.stdin.write(self._pipe_from_string.encode('utf8'))
+
+        _, _ = process.communicate()
+
+        returncode = process.returncode
+
         chdir(previous_directory)
 
         if returncode != 0 and not self._ignore_errors:
@@ -290,6 +341,10 @@ class Command(object):
                 returncode
             ))
 
+    def output(self):
+        """Run command until it finishes and return output as string."""
+        return self.piped.output()
+
     def __str__(self):
         return " ".join(self.arguments)
 
@@ -298,3 +353,6 @@ class Command(object):
 
     def __repr__(self):
         return self.__str__()
+
+
+python = Command(sys.executable)
